clear all
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameters of the system

% sampling times 
Ts = 0.01;                  % samping time of keep-up control
Ts3 = 0.0002;               % sampling period of current control
Tpv = 0.002;                % sampling period of speed and position control


% Arm params
l1 = 0.190;                 % length of arm
h1 = 0.115;                 % heigth of arm
m1 = 0.241;                 % weigth of arm
cg1 = [0 6 15]*10^-3;       % center of gravity of arm
I1 = [2100    -2     0;...
        -2  1285  -404;...
         0  -404   910]*...
      10^(-6);              % momentum of arm     
m1z = 0.1;                  % weigth added to the end of arm


% Pendulum params
l2 = 0.332;                 % length of pendulum
m2 = 0.025;                 % weigth of pendulum
cg2 = [0 132 0.7]*10^-3;    % center of gravity of pendulum
I2 = [470   0   0;...
        0   2   3;...
        0   3 472]*...
      10^(-6);               % momentum of pendulum
m2z = 0.1;                   % weigth added to the end of pendulum

b2 = 0.00005;                % viscous friction in joint between arm and pendulum,
                             % see the influence on system in simulink model RotacniStabilne

% motor params
% mass of rotor is included in the mass of arm
Umax = 24;          % maximal voltage of source
Twinding = 0.010;   % time constant of winding
Rwinding = 1;       % winding resistance
mc = 1/360 *180/pi; % 1V ... 360/s ... 60rpm -> no-load speed 1440rm / 24V
b1 = 0.001;         % viscous friction of rotor [Nm/(/s)]

% input filter params
b = 0.5;            % damping ratio
w = 5;              % natural frequency (rad/s)

Tf = 0.2;           % time const of energy estimation filter
Ts4 = 10*Ts3;       % sampling period of energy estimation discrete filter



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% simulation, produces xMes and uMes
tSim = 170*Ts;
kProv = 0.5;
sim('identification.slx')



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Identification - least squares method

L = [xMes(2:end-1,:),uMes(2:end-1,:)];

Aproc = L\xMes(3:end,:);

% mining the A,B matrixes from the Aproc matrix
% C,D matrixes are chosen so, that output shows all the states (y = x)
B = Aproc(end,:)';
A = Aproc(1:end-1,:);
n = size(A,1);
A = A';
C = eye(n);
D = zeros(n,1);

% simulation, measured u is applied to the identified model
% only measured u is used -: some cumulative error is inevitable
xId = zeros(size(xMes,2),n);
xId(1,:) = xMes(1,:)';
xId(2,:) = xMes(2,:)';
for i = 2:size(xMes,1)
   xId(i+1,:) = A*xId(i,:)' + B*uMes(i); 
end

% plotting
% just for visual assurance

% figure(1)
% hold on
% plot(xId(:,1));
% plot(xMes(:,1)','r*')
% setLegend();
% title('natoceni ramene');
% 
% figure(2)
% hold on
% plot(xId(:,2));
% plot(xMes(:,2)','r*')
% setLegend();
% title('rychlost ramene');
% 
% figure(3)
% hold on
% plot(xId(:,3));
% plot(xMes(:,3)','r*')
% setLegend();
% title('natoceni kyvadla');
% 
% figure(4)
% hold on
% plot(xId(:,4));
% plot(xMes(:,4)','r*')
% setLegend();
% title('rychlost kyvadla');

figure(6)
subplot(4,1,1)
hold on
plot(5:5:size(xMes,1),xMes(5:5:end,1),'r*')
plot(xId(:,1)'')
setLegend();
title('arm angle');
xlabel('i[-]')
ylabel('\alpha[]')

subplot(4,1,2)
hold on
plot(xId(:,2));
plot(5:5:size(xMes,1),xMes(5:5:end,2),'r*')
setLegend();
title('dif. of arm angle');
xlabel('i[-]')
ylabel('\Delta\alpha[]')

subplot(4,1,3)
hold on
plot(xId(:,3));
plot(5:5:size(xMes,1),xMes(5:5:end,3),'r*')
setLegend();
title('pendulum angle');
xlabel('i[-]')
ylabel('\beta[]')

subplot(4,1,4)
hold on
plot(xId(:,4));
plot(5:5:size(xMes,1),xMes(5:5:end,4),'r*')
setLegend();
title('dif. of pendulum angle');
xlabel('i[-]')
ylabel('\Delta\beta[]')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Regulation design (discrete)

% Model of system + state space controler with sumator
n = size(A,1);
X = [A zeros(4,1); ...
-C(1,:), 1];

Y = [B(:,1); ...
0];



% State space controler with sumator 1
% less agresive, meant for catching of pendulum from swing-up phase
evZ1 = 2*[-2 -2.1 -2.2 -2.3 -2.4];
evdZ1 = exp(evZ1*Ts);
Z1 = acker(X, Y, evdZ1)

% State space controler with sumator 2
% more aggresive, meant for quicker folowing of reference signal and
% disturbance rejection
% works in narrower range of states near to pendulum meta-stable position
evZ2 = 3*[-2 -2.1 -2.2 -2.3 -2.4];
evdZ2 = exp(evZ2*Ts);
Z2 = acker(X, Y, evdZ2)

% State space controler with sumator 3
% totaly not agresive, meant for catching of pendulum from swing-up phase
% not used, because the reference folowing is terrible
evZ3 = 1*[-2 -2.1 -2.2 -2.3 -2.4];
evdZ3 = exp(evZ3*Ts);
Z3 = acker(X, Y, evdZ3)


% Simple state space controler, not used
evR = [-10 -30 -31 -32];
evR = [-10 -12 -14 -16];
evdR = exp(evR*Ts);
R = acker(A, B, evdR)


% Some LQR or LQRS design?
% .....



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Swing-up regulator settings

% hard-coded in respective block
% swing1 = 20;
% swing2 = 10;
% swing3 = 8;
% energy1 = -1.5;
% energy2 = -0.7;

changeAngle = 20;
changeAngleHys = 5;

betaInit = 30
tSim = 30
